@@ -27,7 +27,7 @@ namespace GtkSharp.Generation {
 	using System.IO;
 	using System.Xml;
 
-	public class InterfaceGen : ClassBase {
+	public class InterfaceGen : ObjectBase {
 
 		ArrayList vms = new ArrayList ();
 		ArrayList members = new ArrayList ();
@@ -52,6 +52,23 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public override bool ValidateForSubclass ()
+		{
+			ArrayList invalids = new ArrayList ();
+
+			foreach (Method method in methods.Values) {
+				if (!method.Validate ()) {
+					Console.WriteLine ("in type " + QualifiedName);
+					invalids.Add (method);
+				}
+			}
+			foreach (Method method in invalids)
+				methods.Remove (method.Name);
+			invalids.Clear ();
+
+			return base.ValidateForSubclass ();
+		}
+
 		string IfaceName {
 			get {
 				return Name + "Iface";
@@ -171,11 +188,8 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ();
 			sw.WriteLine ("\t\t// signals");
 			foreach (Signal sig in sigs.Values) {
-				if (sig.Validate ()) {
-					sig.GenerateDecl (sw);
-					sig.GenEventHandler (gen_info);
-				} else
-					Console.WriteLine ("of interface " + QualifiedName);
+				sig.GenerateDecl (sw);
+				sig.GenEventHandler (gen_info);
 			}
 		}
 
@@ -204,17 +218,14 @@ namespace GtkSharp.Generation {
 				//if (IgnoreMethod (method))
 					//continue;
 
-				if (method.Validate ()) {
-					if (!vm_decls.Contains (method.Declaration) && method.Name != "GetGType") {
-						if (need_comment) {
-							sw.WriteLine ();
-							sw.WriteLine ("\t\t// non-virtual methods");
-							need_comment = false;
-						}
-						method.GenerateDecl (sw);
+				if (!vm_decls.Contains (method.Declaration) && method.Name != "GetGType") {
+					if (need_comment) {
+						sw.WriteLine ();
+						sw.WriteLine ("\t\t// non-virtual methods");
+						need_comment = false;
 					}
-				} else
-					Console.WriteLine ("of interface " + QualifiedName);
+					method.GenerateDecl (sw);
+				}
 			}
 		}
 
@@ -271,23 +282,19 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ();
 			
 			foreach (Signal sig in sigs.Values) {
-				if (sig.Validate ()) {
-					sig.GenerateDecl (sw);
-					sig.GenEventHandler (gen_info);
-				} else
-					Console.WriteLine ("of interface " + QualifiedName);
+				sig.GenerateDecl (sw);
+				sig.GenEventHandler (gen_info);
 			}
 
 			foreach (Method method in methods.Values) {
 				if (IgnoreMethod (method))
 					continue;
-
-				if (method.Validate ())
-					method.GenerateDecl (sw);
-				else
-					Console.WriteLine ("of interface " + QualifiedName);
+				method.GenerateDecl (sw);
 			}
 
+			foreach (Property prop in props.Values)
+				prop.GenerateDecl (sw, "\t\t");
+
 			AppendCustom (sw, gen_info.CustomDir);
 
 			sw.WriteLine ("\t}");
