@@ -28,7 +28,7 @@ namespace GtkSharp.Generation {
 	using System.Text;
 	using System.Xml;
 
-	public class ObjectGen : ClassBase  {
+	public class ObjectGen : ObjectBase  {
 
 		private ArrayList strings = new ArrayList();
 		private ArrayList vm_nodes = new ArrayList();
@@ -44,7 +44,6 @@ namespace GtkSharp.Generation {
 				XmlElement member = (XmlElement) node;
 
 				switch (node.Name) {
-				case "field":
 				case "callback":
 					Statistics.IgnoreCount++;
 					break;
@@ -72,6 +71,22 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public override bool Validate ()
+		{
+			ArrayList invalids = new ArrayList ();
+
+			foreach (ChildProperty prop in childprops.Values) {
+				if (!prop.Validate ()) {
+					Console.WriteLine ("in Object " + QualifiedName);
+					invalids.Add (prop);
+				}
+			}
+			foreach (ChildProperty prop in invalids)
+				childprops.Remove (prop);
+
+			return base.Validate ();
+		}
+
 		private bool DisableVoidCtor {
 			get {
 				return Elem.HasAttribute ("disable_void_ctor");
@@ -113,11 +128,13 @@ namespace GtkSharp.Generation {
 			
 			return result;
 		}
- 
+
 		public override void Generate (GenerationInfo gen_info)
 		{
-			DirectoryInfo di = GetDirectoryInfo (gen_info.Dir, gen_info.AssemblyName);
-			di.objects.Add (CName, QualifiedName);
+			gen_info.CurrentType = Name;
+
+			string asm_name = gen_info.AssemblyName.Length == 0 ? NS.ToLower () + "-sharp" : gen_info.AssemblyName;
+			DirectoryInfo di = GetDirectoryInfo (gen_info.Dir, asm_name);
 
 			StreamWriter sw = gen_info.Writer = gen_info.OpenStream (Name);
 
@@ -135,24 +152,30 @@ namespace GtkSharp.Generation {
 				sw.WriteLine ("\t[Obsolete]");
 			sw.Write ("\tpublic {0} class " + Name, IsAbstract ? "abstract" : "");
 			string cs_parent = table.GetCSType(Elem.GetAttribute("parent"));
-			if (cs_parent != "")
+			if (cs_parent != "") {
+				di.objects.Add (CName, QualifiedName);
 				sw.Write (" : " + cs_parent);
-			if (interfaces != null) {
-				foreach (string iface in interfaces) {
-					if (Parent != null && Parent.Implements (iface))
-						continue;
-					sw.Write (", " + table.GetCSType (iface));
-				}
+			}
+			foreach (string iface in interfaces) {
+				if (Parent != null && Parent.Implements (iface))
+					continue;
+				sw.Write (", " + table.GetCSType (iface));
+			}
+			foreach (string iface in managed_interfaces) {
+				if (Parent != null && Parent.Implements (iface))
+					continue;
+				sw.Write (", " + iface);
 			}
 			sw.WriteLine (" {");
 			sw.WriteLine ();
 
 			GenCtors (gen_info);
-			GenProperties (gen_info);
+			GenProperties (gen_info, null);
+			GenFields (gen_info);
 			GenChildProperties (gen_info);
 			
 			bool has_sigs = (sigs != null && sigs.Count > 0);
-			if (!has_sigs && interfaces != null) {
+			if (!has_sigs) {
 				foreach (string iface in interfaces) {
 					ClassBase igen = table.GetClassGen (iface);
 					if (igen != null && igen.Signals != null) {
@@ -177,16 +200,19 @@ namespace GtkSharp.Generation {
 
 			GenMethods (gen_info, null, null);
 			
-			if (interfaces != null) {
+			if (interfaces.Count != 0) {
 				Hashtable all_methods = new Hashtable ();
+				foreach (Method m in Methods.Values)
+					all_methods[m.Name] = m;
 				Hashtable collisions = new Hashtable ();
 				foreach (string iface in interfaces) {
 					ClassBase igen = table.GetClassGen (iface);
 					foreach (Method m in igen.Methods.Values) {
-						if (all_methods.Contains (m.Name))
+						Method collision = all_methods[m.Name] as Method;
+						if (collision != null && collision.Signature.Types == m.Signature.Types)
 							collisions[m.Name] = true;
 						else
-							all_methods[m.Name] = true;
+							all_methods[m.Name] = m;
 					}
 				}
 					
@@ -195,6 +221,7 @@ namespace GtkSharp.Generation {
 						continue;
 					ClassBase igen = table.GetClassGen (iface);
 					igen.GenMethods (gen_info, collisions, this);
+					igen.GenProperties (gen_info, this);
 					igen.GenSignals (gen_info, this);
 				}
 			}
@@ -204,11 +231,11 @@ namespace GtkSharp.Generation {
 				sw.WriteLine (" {\n\t\t\t get { return \"" + str.GetAttribute ("value") + "\"; }\n\t\t}");
 			}
 
-			if (GetExpected (CName) != (QualifiedName + "," + gen_info.AssemblyName)) {
+			if (cs_parent != String.Empty && GetExpected (CName) != QualifiedName) {
 				sw.WriteLine ();
 				sw.WriteLine ("\t\tstatic " + Name + " ()");
 				sw.WriteLine ("\t\t{");
-				sw.WriteLine ("\t\t\tGtkSharp." + Studlify (gen_info.AssemblyName) + ".ObjectManager.Initialize ();");
+				sw.WriteLine ("\t\t\tGtkSharp." + Studlify (asm_name) + ".ObjectManager.Initialize ();");
 				sw.WriteLine ("\t\t}");
 			}
 
@@ -257,16 +284,17 @@ namespace GtkSharp.Generation {
 
 			StreamWriter sw = gen_info.Writer;
 
-			sw.WriteLine ("\t\tpublic class " + Name + "Child : Gtk.Container.ContainerChild {");
+			ObjectGen child_ancestor = Parent as ObjectGen;
+			while (child_ancestor.CName != "GtkContainer" &&
+			       child_ancestor.childprops.Count == 0)
+				child_ancestor = child_ancestor.Parent as ObjectGen;
+
+			sw.WriteLine ("\t\tpublic class " + Name + "Child : " + child_ancestor.NS + "." + child_ancestor.Name + "." + child_ancestor.Name + "Child {");
 			sw.WriteLine ("\t\t\tprotected internal " + Name + "Child (Gtk.Container parent, Gtk.Widget child) : base (parent, child) {}");
 			sw.WriteLine ("");
 
-			foreach (ChildProperty prop in childprops.Values) {
-				if (prop.Validate ())
-					prop.Generate (gen_info, "\t\t\t");
-				else
-					Console.WriteLine("in Object " + QualifiedName);
-			}
+			foreach (ChildProperty prop in childprops.Values)
+				prop.Generate (gen_info, "\t\t\t", null);
 
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ("");
@@ -313,35 +341,25 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		/* Keep this in sync with the one in glib/ObjectManager.cs */
+		/* Keep this in sync with the one in glib/GType.cs */
 		private static string GetExpected (string cname)
 		{
-			StringBuilder expected = new StringBuilder ();
-			string ns = "";
-			bool needs_dot = true;
-			for (int i = 0; i < cname.Length; i++)
-			{
-				if (needs_dot && i > 0 && Char.IsUpper (cname[i])) {
-					if (expected.Length == 1 && expected[0] == 'G') {
-						ns = "glib";
-						expected = new StringBuilder ("GLib.");
-					} else {
-						ns = expected.ToString ().ToLower ();
-						expected.Append ('.');
-					}
-					needs_dot = false;
+			for (int i = 1; i < cname.Length; i++) {
+				if (Char.IsUpper (cname[i])) {
+					if (i == 1 && cname[0] == 'G')
+						return "GLib." + cname.Substring (1);
+					else
+						return cname.Substring (0, i) + "." + cname.Substring (i);
 				}
-				expected.Append (cname[i]);
 			}
-			expected.AppendFormat (",{0}-sharp", ns);
 
-			return expected.ToString ();
+			throw new ArgumentException ("cname doesn't follow the NamespaceType capitalization style: " + cname);
 		}
 
 		private static bool NeedsMap (Hashtable objs, string assembly_name)
 		{
 			foreach (string key in objs.Keys)
-				if (GetExpected (key) != ((string) objs[key] + "," + assembly_name))
+				if (GetExpected (key) != ((string) objs[key]))
 					return true;
 			
 			return false;
@@ -391,8 +409,8 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t\tinitialized = true;");
 	
 			foreach (string key in dir_info.objects.Keys) {
-				if (GetExpected(key) != ((string) dir_info.objects[key] + "," + dir_info.assembly_name))
-					sw.WriteLine ("\t\t\tGLib.ObjectManager.RegisterType(\"" + key + "\", \"" + dir_info.objects [key] + "," + dir_info.assembly_name + "\");");
+				if (GetExpected(key) != ((string) dir_info.objects[key]))
+					sw.WriteLine ("\t\t\tGLib.GType.Register ({0}.GType, typeof ({0}));", dir_info.objects [key]);
 			}
 					
 			sw.WriteLine ("\t\t}");
